"use strict";
((window, notification) => {
    const notifications = new Map();
    // Handle events sent from the browser process
    window.addEventListener('message', ({ data: { type, data } }) => {
        if (type === 'notification-callback') {
            const { callbackName, id } = data;
            const notification = notifications.get(id);
            if (!notification) {
                return;
            }
            if (notification[callbackName]) {
                notification[callbackName]();
            }
            if (callbackName === 'onclose') {
                notifications.delete(id);
            }
        }
        if (type === 'notification-reply-callback') {
            const { callbackName, id, previousConversation, reply } = data;
            const notification = notifications.get(id);
            if (!notification) {
                return;
            }
            if (notification[callbackName]) {
                notification[callbackName]();
            }
            notifications.delete(id);
            window.postMessage({ type: 'notification-reply', data: { previousConversation, reply } }, '*');
        }
    });
    let counter = 1;
    const augmentedNotification = Object.assign(class {
        _id;
        constructor(title, options) {
            // According to https://github.com/sindresorhus/caprine/pull/637, the Notification
            // constructor can be called with non-string title and body.
            let { body } = options;
            const bodyProperties = body.props;
            body = bodyProperties ? bodyProperties.content[0] : options.body;
            const titleProperties = title.props;
            title = titleProperties ? titleProperties.content[0] : title;
            this._id = counter++;
            notifications.set(this._id, this);
            window.postMessage({
                type: 'notification',
                data: {
                    title,
                    id: this._id,
                    ...options,
                    body,
                },
            }, '*');
        }
        // No-op, but Messenger expects this method to be present
        close() { } // eslint-disable-line @typescript-eslint/no-empty-function
    }, notification);
    Object.assign(window, { notification: augmentedNotification });
})(window, Notification);
